@@ -5,14 +5,16 @@
 
 This implements the python client for the ONNX model hub.
 """
+
+from __future__ import annotations
+
 import hashlib
 import json
 import os
 import sys
-import tarfile
 from io import BytesIO
 from os.path import join
-from typing import IO, Any, Dict, List, Optional, Set, Tuple, cast
+from typing import IO, Any, cast
 from urllib.error import HTTPError
 from urllib.request import urlopen
 
@@ -27,8 +29,7 @@ else:
 
 
 class ModelInfo:
-    """
-    A class to represent a model's property and metadata in the ONNX Hub.
+    """A class to represent a model's property and metadata in the ONNX Hub.
     It extracts model name, path, sha, tags, etc. from the passed in raw_model_info dict.
 
     Attributes:
@@ -40,25 +41,26 @@ class ModelInfo:
         opset: The opset version of the model.
     """
 
-    def __init__(self, raw_model_info: Dict[str, Any]) -> None:
-        """
-        Parameters:
+    def __init__(self, raw_model_info: dict[str, Any]) -> None:
+        """Initializer.
+
+        Args:
             raw_model_info: A JSON dict containing the model info.
         """
         self.model = cast(str, raw_model_info["model"])
 
         self.model_path = cast(str, raw_model_info["model_path"])
-        self.metadata: Dict[str, Any] = cast(Dict[str, Any], raw_model_info["metadata"])
-        self.model_sha: Optional[str] = None
+        self.metadata: dict[str, Any] = cast(dict[str, Any], raw_model_info["metadata"])
+        self.model_sha: str | None = None
         if "model_sha" in self.metadata:
             self.model_sha = cast(str, self.metadata["model_sha"])
 
-        self.tags: Set[str] = set()
+        self.tags: set[str] = set()
         if "tags" in self.metadata:
-            self.tags = set(cast(List[str], self.metadata["tags"]))
+            self.tags = set(cast(list[str], self.metadata["tags"]))
 
         self.opset = cast(int, raw_model_info["opset_version"])
-        self.raw_model_info: Dict[str, Any] = raw_model_info
+        self.raw_model_info: dict[str, Any] = raw_model_info
 
     def __str__(self) -> str:
         return f"ModelInfo(model={self.model}, opset={self.opset}, path={self.model_path}, metadata={self.metadata})"
@@ -68,28 +70,26 @@ class ModelInfo:
 
 
 def set_dir(new_dir: str) -> None:
-    """
-    Sets the current ONNX hub cache location
+    """Sets the current ONNX hub cache location.
 
-    :param new_dir: location of new model hub cache
+    Args:
+        new_dir: Location of new model hub cache.
     """
     global _ONNX_HUB_DIR  # noqa: PLW0603
     _ONNX_HUB_DIR = new_dir
 
 
 def get_dir() -> str:
-    """
-    Gets the current ONNX hub cache location
+    """Gets the current ONNX hub cache location.
 
-    :return: The location of the ONNX hub model cache
+    Returns:
+        The location of the ONNX hub model cache.
     """
     return _ONNX_HUB_DIR
 
 
-def _parse_repo_info(repo: str) -> Tuple[str, str, str]:
-    """
-    Gets the repo owner, name and ref from a repo specification string.
-    """
+def _parse_repo_info(repo: str) -> tuple[str, str, str]:
+    """Gets the repo owner, name and ref from a repo specification string."""
     repo_owner = repo.split(":")[0].split("/")[0]
     repo_name = repo.split(":")[0].split("/")[1]
     if ":" in repo:
@@ -100,8 +100,7 @@ def _parse_repo_info(repo: str) -> Tuple[str, str, str]:
 
 
 def _verify_repo_ref(repo: str) -> bool:
-    """
-    Verifies whether the given model repo can be trusted.
+    """Verifies whether the given model repo can be trusted.
     A model repo can be trusted if it matches onnx/models:main.
     """
     repo_owner, repo_name, repo_ref = _parse_repo_info(repo)
@@ -109,13 +108,16 @@ def _verify_repo_ref(repo: str) -> bool:
 
 
 def _get_base_url(repo: str, lfs: bool = False) -> str:
-    """
-    Gets the base github url from a repo specification string
+    """Gets the base github url from a repo specification string.
 
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param lfs: whether the url is for downloading lfs models
-    :return: the base github url for downloading
+    Args:
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main".
+        lfs: Whether the url is for downloading lfs models.
+
+    Returns:
+        The base github url for downloading.
     """
     repo_owner, repo_name, repo_ref = _parse_repo_info(repo)
 
@@ -125,11 +127,11 @@ def _get_base_url(repo: str, lfs: bool = False) -> str:
 
 
 def _download_file(url: str, file_name: str) -> None:
-    """
-    Downloads the file with specifed file_name from the url
+    """Downloads the file with specified file_name from the url.
 
-    :param url: a url of download link
-    :param file_name: a specified file name for the downloaded file
+    Args:
+        url: A url of download link.
+        file_name: A specified file name for the downloaded file.
     """
     chunk_size = 16384  # 1024 * 16
     with urlopen(url) as response, open(file_name, "wb") as f:
@@ -143,23 +145,28 @@ def _download_file(url: str, file_name: str) -> None:
 
 def list_models(
     repo: str = "onnx/models:main",
-    model: Optional[str] = None,
-    tags: Optional[List[str]] = None,
-) -> List[ModelInfo]:
-    """
-    Gets the list of model info consistent with a given name and tags
+    model: str | None = None,
+    tags: list[str] | None = None,
+) -> list[ModelInfo]:
+    """Gets the list of model info consistent with a given name and tags
 
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param model: The name of the model to search for. If `None`, will return all models with matching tags.
-    :param tags: A list of tags to filter models by. If `None`, will return all models with matching name.
-    :return: list of ModelInfo
+    Args:
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        model: The name of the model to search for. If `None`, will
+            return all models with matching tags.
+        tags: A list of tags to filter models by. If `None`, will return
+            all models with matching name.
+
+    Returns:
+        ``ModelInfo``s.
     """
     base_url = _get_base_url(repo)
     manifest_url = base_url + "ONNX_HUB_MANIFEST.json"
     try:
         with urlopen(manifest_url) as response:
-            manifest: List[ModelInfo] = [
+            manifest: list[ModelInfo] = [
                 ModelInfo(info) for info in json.load(cast(IO[str], response))
             ]
     except HTTPError as e:
@@ -177,7 +184,7 @@ def list_models(
         return matching_models
 
     canonical_tags = {t.lower() for t in tags}
-    matching_info_list: List[ModelInfo] = []
+    matching_info_list: list[ModelInfo] = []
     for m in matching_models:
         model_tags = {t.lower() for t in m.tags}
         if len(canonical_tags.intersection(model_tags)) > 0:
@@ -186,16 +193,21 @@ def list_models(
 
 
 def get_model_info(
-    model: str, repo: str = "onnx/models:main", opset: Optional[int] = None
+    model: str, repo: str = "onnx/models:main", opset: int | None = None
 ) -> ModelInfo:
-    """
-    Gets the model info matching the given name and opset.
+    """Gets the model info matching the given name and opset.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to get. The default of `None` will return the model with largest opset.
-    :return: ModelInfo
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to get. The default of `None` will
+            return the model with largest opset.
+
+    Returns:
+        ``ModelInfo``.
     """
     matching_models = list_models(repo, model)
     if not matching_models:
@@ -216,27 +228,34 @@ def get_model_info(
 def load(
     model: str,
     repo: str = "onnx/models:main",
-    opset: Optional[int] = None,
+    opset: int | None = None,
     force_reload: bool = False,
     silent: bool = False,
-) -> Optional[onnx.ModelProto]:
-    """
-    Downloads a model by name from the onnx model hub
+) -> onnx.ModelProto | None:
+    """Downloads a model by name from the onnx model hub.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: ModelProto or None
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        ModelProto or None
     """
     selected_model = get_model_info(model, repo, opset)
     local_model_path_arr = selected_model.model_path.split("/")
     if selected_model.model_sha is not None:
-        local_model_path_arr[
-            -1
-        ] = f"{selected_model.model_sha}_{local_model_path_arr[-1]}"
+        local_model_path_arr[-1] = (
+            f"{selected_model.model_sha}_{local_model_path_arr[-1]}"
+        )
     local_model_path = join(_ONNX_HUB_DIR, os.sep.join(local_model_path_arr))
 
     if force_reload or not os.path.exists(local_model_path):
@@ -274,20 +293,28 @@ def load(
 def download_model_with_test_data(
     model: str,
     repo: str = "onnx/models:main",
-    opset: Optional[int] = None,
+    opset: int | None = None,
     force_reload: bool = False,
     silent: bool = False,
-) -> Optional[str]:
-    """
-    Downloads a model along with test data by name from the onnx model hub and returns the directory to which the files have been extracted.
+) -> str | None:
+    """Downloads a model along with test data by name from the onnx model hub and returns the directory to which the files have been extracted.
+    Users are responsible for making sure the model comes from a trusted source, and the data is safe to be extracted.
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: str or None
+    Args:
+        model: The name of the onnx model in the manifest. This field is
+            case-sensitive
+        repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        str or None
     """
     selected_model = get_model_info(model, repo, opset)
 
@@ -298,9 +325,9 @@ def download_model_with_test_data(
     model_with_data_sha = selected_model.metadata["model_with_data_sha"]
 
     if model_with_data_sha is not None:
-        local_model_with_data_path_arr[
-            -1
-        ] = f"{model_with_data_sha}_{local_model_with_data_path_arr[-1]}"
+        local_model_with_data_path_arr[-1] = (
+            f"{model_with_data_sha}_{local_model_with_data_path_arr[-1]}"
+        )
     local_model_with_data_path = join(
         _ONNX_HUB_DIR, os.sep.join(local_model_with_data_path_arr)
     )
@@ -337,12 +364,12 @@ def download_model_with_test_data(
                 "download the model from the model hub."
             )
 
-    with tarfile.open(local_model_with_data_path) as model_with_data_zipped:
-        # FIXME: Avoid index manipulation with magic numbers
-        local_model_with_data_dir_path = local_model_with_data_path[
-            0 : len(local_model_with_data_path) - 7
-        ]
-        model_with_data_zipped.extractall(local_model_with_data_dir_path)
+    local_model_with_data_dir_path = local_model_with_data_path[
+        0 : len(local_model_with_data_path) - len(".tar.gz")
+    ]
+    onnx.utils._extract_model_safe(
+        local_model_with_data_path, local_model_with_data_dir_path
+    )
     model_with_data_path = (
         local_model_with_data_dir_path
         + "/"
@@ -357,21 +384,31 @@ def load_composite_model(
     preprocessing_model: str,
     network_repo: str = "onnx/models:main",
     preprocessing_repo: str = "onnx/models:main",
-    opset: Optional[int] = None,
+    opset: int | None = None,
     force_reload: bool = False,
     silent: bool = False,
-) -> Optional[onnx.ModelProto]:
-    """
-    Builds a composite model including data preprocessing by downloading a network and a preprocessing model
+) -> onnx.ModelProto | None:
+    """Builds a composite model including data preprocessing by downloading a network and a preprocessing model
     and combine it into a single model
 
-    :param model: The name of the onnx model in the manifest. This field is case-sensitive
-    :param repo: The location of the model repo in format "user/repo[:branch]".
-        If no branch is found will default to "main"
-    :param opset: The opset of the model to download. The default of `None` automatically chooses the largest opset
-    :param force_reload: Whether to force the model to re-download even if its already found in the cache
-    :param silent: Whether to suppress the warning message if the repo is not trusted.
-    :return: ModelProto or None
+    Args:
+        network_model: The name of the onnx model in the manifest.
+        preprocessing_model: The name of the preprocessing model.
+        network_repo: The location of the model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        preprocessing_repo: The location of the proprocessing model repo in format
+            "user/repo[:branch]". If no branch is found will default to
+            "main"
+        opset: The opset of the model to download. The default of `None`
+            automatically chooses the largest opset
+        force_reload: Whether to force the model to re-download even if
+            its already found in the cache
+        silent: Whether to suppress the warning message if the repo is
+            not trusted.
+
+    Returns:
+        ModelProto or None
     """
     preprocessing = load(
         preprocessing_model, preprocessing_repo, opset, force_reload, silent
@@ -384,9 +421,9 @@ def load_composite_model(
     if network is None:
         raise RuntimeError(f"Could not load the network model: {network_model}")
 
-    all_domains: Set[str] = set()
-    domains_to_version_network: Dict[str, int] = {}
-    domains_to_version_preprocessing: Dict[str, int] = {}
+    all_domains: set[str] = set()
+    domains_to_version_network: dict[str, int] = {}
+    domains_to_version_preprocessing: dict[str, int] = {}
 
     for opset_import_entry in network.opset_import:
         domain = (
